// let chart = null;

let chartData = {
    labels: [],
    periods: null,
};

function requestChartData(id, topic, from, to, place) {
    /* Запрашивает данные потребления как по локации,
    так и по отдельным потребителям в световой зоне
     */
    if (!['location', 'area'].includes(place)) {
        throw 'Неверное значение параметра place: ' + place + '. Укажите "location" или "area"';

    }
    chartData.periods = chartData.periods ?? calculatePeriods(id, from, to);
    const algo = "integral";

    chartData.periods.forEach(function(period, i) {
        client.send(topic,
            JSON.stringify({
                    value: {
                        algorithm: algo,
                        startTime: period.from,
                        endTime: period.to,
                    },
                    key: [id, i, period.label, place+"_chart_data"].join('#'),
                    session: broker.guid
                }
            ));
    });
}

function calculatePeriods(id, fromDate, toDate) {
    let toDateObj = moment(toDate);
    let delta = moment.duration(toDateObj.diff(fromDate));
    let nowObj = moment();
    if (toDateObj.isAfter(nowObj)) {
        toDateObj = nowObj;
    }
    //TODO формат зависит от локали
    let quant = moment.duration(1, 'days');
    let format = 'D MMM';
    if (delta.asDays() < 2) {
        quant = moment.duration(1, 'hours');
        format = 'H:mm';
    } else if (delta.asDays() <= 31) {
        // оставим параметры по умолчанию и выйдем
    } else if (delta.asDays() < 70) {
        quant = moment.duration(3, 'days');
    } else if (delta.asWeeks() < 52) {
        quant = moment.duration(1, 'weeks');
    } else {
        quant = moment.duration(1, 'months');
        format = "MMM'YY";
    }
    let periods = [];
    let from = moment(fromDate);
    do {
        periods.push({from: from.format(), label: from.format(format), to: from.add(quant).format()});  // from.add() меняет сам объект
    } while (from.isSameOrBefore(toDateObj))
    // С прибавлением последнего кванта мы можем перевалить за границу интервала, поэтому обрежем последний период
    periods[periods.length-1].to = toDateObj.format();

    return periods
}

function drawConsumptionChart(id) {

    let ctx = document.getElementById('chart-' + id);
    // при изменении периода удалим старый график, чтоб на его месте построить новый
    let chart = null
    chart?.destroy();
    // TODO chart.update()

    // Draw
    if (chartData[id]?.awada.data.length > 0) {
        let data = {labels: chartData.labels, datasets: [chartData[id].awada, chartData[id].withoutAwada]};
        chart = new Chart(ctx, {type: 'line', data: data});
    } else {
        // удаляем весь <div> с графиком
        deleteChartContainer(id);
    }
    // Считаем и отображаем экономию
    let cons_elem = document.getElementById("consumption-" + id);
    let cons = document
        .querySelector('[total-power-' + id + ']')
        .textContent;

    cons_elem.textContent = cons + ' ' + gettext('consumption_energy_unit');
    document.getElementById('economy-' + id).textContent = calculateSaving(id) + ' %';
    delete chartData[String(id)];
}

function processChartData(key, data) {
    let [id, index, label, place] = key.split('#');
    // вычислили число периодов, столько точек будем ждать из брокера
    chartData[id].valueCount = chartData[id]?.valueCount || chartData.periods.length * (chartData[id]?.partCount || 1);

    if (Object.keys(data).length === 0) {
        // нет данных, точку не обрабатываем
        if (chartData[id].valueCount) {
            --chartData[id].valueCount;
        }
        return;
    }
    if (index % 2 != 0) {
        label = '';
    }
    if (!chartData.labels[index]) {
        chartData.labels[index] = label;
    }

    let newValue = parseFloat(chartData[id].awada.data[index] || 0.0) + (data.value/1000 || 0.0);
    chartData[id].awada.data[index] = +newValue.toFixed(1);
    chartData[id].withoutAwada.data[index] = consumptionWithoutAwada(id, index, data, place.includes('area'));

    --chartData[id].valueCount;
    if (chartData[id].valueCount <= 0) {
        drawConsumptionChart(id);
    }
}

function clearEmptyData(list) {
    //TODO разобраться, почему глючат графики
    // while (list[0] === undefined) {
    //    list.shift();
    // }
    return list;
}

function consumptionWithoutAwada(id, index, data, isArea = false) {
    let totalPower = +document
        .getElementById(id)
        .getAttribute("total-power-" + id) / 1000;
    if (!totalPower) {
        return 0.0;
    }
    let hours = ((data.workTime || 0.0) + (data.idleTime || 0.0)) / (60 * 60 * 1000);
    // для световой зоны определяем, является ли текущее значение максимальным из всех потребителей
    if (isArea) {
        hours = chartData[id].hoursWithoutAwada[index] = Math.max(hours, chartData[id].hoursWithoutAwada[index] || 0);
    }
    return +(totalPower * hours).toFixed(1);
}

function resetChartData(id=null, partCount=1) {
    if (id) {
        chartData[id] = {
            valueCount: 0,  // число число значений, ожидаемых для построения графика
            partCount: partCount,  // число слагаемых в каждой точке (напр. светильников в световой зоне)
            withoutAwada: {borderColor: '#A700FF', backgroundColor: '#A700FF', data: []},
            awada: {borderColor: '#B1B9FF', backgroundColor: '#B1B9FF', data: []},
            hoursWithoutAwada: []  // массив для хранения максимального из времен работы потребителей в световой зоне
        };
    } else {
        chartData = {
            labels: [],
            periods: null,
        };
    }

}

function calculateSaving(id) {
    const noAwadaValues = chartData[id].withoutAwada.data;
    const noAwadaConsumption = noAwadaValues.reduce(function (total, item) {
        return total + item;
    }, 0);
    if (noAwadaConsumption == 0) {
        return 0;
    }
    let totalPower = +document
        .getElementById(id)
        .textContent;
    return ((1 - (+totalPower / noAwadaConsumption))*100).toFixed(1);
}

function formatChart(chartObject) {
    chartObject.setLineStyle('straight');
    chartObject.setPadding(30, 10, 30, 10);
    chartObject.setTitleShow(false);
    chartObject.setLegendTableShow(false);
    chartObject.setTooltipShow(false);
    // if (chartObject.getMaxValue() < 2) {
    //     chartObject.setGridYInterval(0.1);
    // }
    chartObject.setGridXShow(true);
    chartObject.setGridYShow(true);
    chartObject.setAnimateShow(false);
    chartObject.setValueOnChartShow(true);
}

function deleteChartContainer(id) {
    $('#chart-container-' + id).remove();
}
